#!/usr/bin/env python
"""
_DQMJobsByStatus_

T0Mon: this class is used in the detail pages 
"""

from T0.DAS.Database.Oracle.LongResultQuery import LongResultQuery
from WMCore.Database.DBFormatter import DBFormatter

class DQMJobsByStatus(LongResultQuery):
    """
    retrieve job stats
    
    """
    sql="""SELECT wmbs_job.id AS job_id,
                        wmbs_job.state_time AS state_time,
                        wmbs_file_runlumi_map.run AS run_id,
                        primary_dataset.name AS dataset,
                        CASE  
                          WHEN stream_special_dataset_assoc.stream_id IS NULL THEN 'BULK'
                          ELSE 'EXPRESS'
                        END AS status,
                        wmbs_fileset.name as fileset,
                        COUNT(*) AS inputfiles
                 FROM wmbs_job
                 INNER JOIN wmbs_jobgroup ON
                   wmbs_jobgroup.id = wmbs_job.jobgroup 
                 INNER JOIN wmbs_subscription ON
                   wmbs_jobgroup.subscription = wmbs_subscription.id
                 INNER JOIN  wmbs_fileset ON
                    wmbs_subscription.fileset = wmbs_fileset.id
                 INNER JOIN wmbs_job_assoc ON
                   wmbs_job_assoc.job = wmbs_job.id
                 INNER JOIN wmbs_file_dataset_path_assoc ON
                   wmbs_file_dataset_path_assoc.file_id = wmbs_job_assoc.fileid
                 INNER JOIN dataset_path ON
                   dataset_path.id = wmbs_file_dataset_path_assoc.dataset_path_id
                 INNER JOIN primary_dataset ON
                   primary_dataset.id = dataset_path.primary_dataset
                 INNER JOIN data_tier ON
                   data_tier.id = dataset_path.data_tier AND
                   data_tier.name = 'DQM'
                 LEFT OUTER JOIN stream_special_dataset_assoc ON
                   stream_special_dataset_assoc.primary_dataset_id = dataset_path.primary_dataset
                 INNER JOIN wmbs_file_runlumi_map ON
                   wmbs_file_runlumi_map.fileid = wmbs_job_assoc.fileid 
                 
                 WHERE 
                    wmbs_fileset.name != 'Mergeable' AND
                    wmbs_fileset.name != 'ExpressMergeable'
                 %s
                 %s
                 GROUP BY wmbs_job.id,
                          wmbs_job.state_time,
                          wmbs_file_runlumi_map.run,
                          primary_dataset.name,
                          wmbs_fileset.name,
                          CASE  
                            WHEN stream_special_dataset_assoc.stream_id IS NULL THEN 'BULK'
                            ELSE 'EXPRESS'
                          END
                 ORDER BY wmbs_job.id DESC
                 """
    def execute(self, run, status, uplimit, downlimit, conn = None, transaction = False):
        """
        create correct sql query and execute it
         
        """
        try:
            run = int(run)
        except ValueError:
            run = 0

        allowedStates = [ "acquired", "complete", "failed" ]
        try:
            status = str(status)
            status = status.lower()
            if status not in allowedStates:
                raise ValueError
        except ValueError:
            return { "Error" : "wrong status name", "hint" : allowedStates }
        
        bindVars = { 'uplimit' : uplimit,
                     'downlimit' : downlimit }

        if run > 0:
            sqlRunCheck = """AND wmbs_file_runlumi_map.run = :RUN"""
            bindVars.update( { 'RUN' : run } )
        else:
            sqlRunCheck = ""

        if status == allowedStates[0]:
            sqlStatusCheck = """AND wmbs_job.state != (SELECT id FROM wmbs_job_state WHERE name = 'cleanout')"""
        elif status == allowedStates[1]:
            sqlStatusCheck = """AND wmbs_job.state = (SELECT id FROM wmbs_job_state WHERE name = 'cleanout')
                                AND wmbs_job.outcome = 1"""
        elif status == allowedStates[2]:
            sqlStatusCheck = """AND wmbs_job.state = (SELECT id FROM wmbs_job_state WHERE name = 'cleanout')
                                AND wmbs_job.outcome = 0"""

        sql = self.sql % (sqlRunCheck,
                        sqlStatusCheck)

        result = self.dbi.processData(sql, bindVars, conn = conn,
                                      transaction = transaction)
        
        return self.formatDict(result)

class DQMJobsByStatus_count(DBFormatter):
    """
    retrieve job count
    
    """

    def execute(self, run, status, conn = None, transaction = False):
        """
        create correct sql query and execute it
         
        """
        try:
            run = int(run)
        except ValueError:
            run = 0

        allowedStates = [ "acquired", "complete", "failed" ]
        try:
            status = str(status)
            status = status.lower()
            if status not in allowedStates:
                raise ValueError
        except ValueError:
            return { "Error" : "wrong status name", "hint" : allowedStates }
        
        bindVars={}
        #bindVars = { 'FILESET' : self.fileset }

        if run > 0:
            sqlRunCheck = """AND wmbs_file_runlumi_map.run = :RUN"""
            bindVars.update( { 'RUN' : run } )
        else:
            sqlRunCheck = ""

        if status == allowedStates[0]:
            sqlStatusCheck = """WHERE wmbs_job.state != (SELECT id FROM wmbs_job_state WHERE name = 'cleanout')"""
        elif status == allowedStates[1]:
            sqlStatusCheck = """WHERE wmbs_job.state = (SELECT id FROM wmbs_job_state WHERE name = 'cleanout')
                                AND wmbs_job.outcome = 1"""
        elif status == allowedStates[2]:
            sqlStatusCheck = """WHERE wmbs_job.state = (SELECT id FROM wmbs_job_state WHERE name = 'cleanout')
                                AND wmbs_job.outcome = 0"""

        sql = """SELECT COUNT(DISTINCT wmbs_job.id)
                 FROM wmbs_job
                 INNER JOIN wmbs_jobgroup ON
                   wmbs_jobgroup.id = wmbs_job.jobgroup AND
                   wmbs_jobgroup.subscription != (SELECT id FROM wmbs_subscription
                                                  WHERE fileset = (SELECT id FROM wmbs_fileset
                                                                   WHERE name = 'Mergeable')) AND
                   wmbs_jobgroup.subscription != (SELECT id FROM wmbs_subscription
                                                  WHERE fileset = (SELECT id FROM wmbs_fileset
                                                                   WHERE name = 'ExpressMergeable'))
                 INNER JOIN wmbs_job_assoc ON
                   wmbs_job_assoc.job = wmbs_job.id
                 INNER JOIN wmbs_file_dataset_path_assoc ON
                   wmbs_file_dataset_path_assoc.file_id = wmbs_job_assoc.fileid
                 INNER JOIN dataset_path ON
                   dataset_path.id = wmbs_file_dataset_path_assoc.dataset_path_id
                 INNER JOIN data_tier ON
                   data_tier.id = dataset_path.data_tier AND
                   data_tier.name = 'DQM'
                 INNER JOIN wmbs_file_runlumi_map ON
                   wmbs_file_runlumi_map.fileid = wmbs_job_assoc.fileid 
                 %s
                 %s
                 """ % (sqlRunCheck,
                        sqlStatusCheck)

        result = self.dbi.processData(sql, bindVars, conn = conn,
                                      transaction = transaction)
        
        return self.format(result)
